;
;	mkmfm.com [1]	... making MFM sigmnal  ver 1.0
;	   get 16 bits MFM code from 8 bits Data
;		exp. 0010 1010 -> 0010 0010 0100 0100
;		       2    A  ->   2    4    4    4
;	   if option [1] added then add 1st clock
;		     0010 1010 -> 1010 0010 0100 0100
;		       2    A  ->   A    4    4    4
;
;		start	2023.12.10
;		final	2023.12.13

;				by GarakutaEn
;--------------------------------------
code	segment
	assume	cs:code,es:code,ss:code
	org	100h
;--------------------------------------
;ascii codes
bel	equ	07h	;BEL
cr	equ	0dh	;carriage return
lf	equ	0ah	;line feed
esk	equ	1bh	;ESC key

;dos int services
rdky	equ	01h	;wait key = al
wrch	equ	02h	;write a char = dl
;drky	equ	07h	;direct key check without echo
;rdch	equ	08h	;read a char = al
;wrln	equ	09h	;write a line = ds:dx
;rdln	equ	0ah	;read line = ds:dx
;gdrv	equ	19h	;bootstrap loader

;svct	equ	25h	;ds:dx
;gvct	equ	35h	;es:bx

;srtn	equ	31h	;return, but keep process dx / exe
nrtn	equ	4ch	;normal return to dos

;gtmem	equ	48h	;bx
;rlmem	equ	49h	;es paragrph

;mkfl	equ	3ch	;make a file = ds:dx, cx
;opfl	equ	3dh	;open the file = ds:dx, al
;clfl	equ	3eh	;close the file = bx
;rdfl	equ	3fh	;read from the file = ds:dx, bx, cx
;wrfl	equ	40h	;write to the file = ds:dx, bx, cx

;rom int services
;inky	equ	00h
;ckky	equ	01h	;if key, zero_flag = 0 ; al = ASCII, ah = ID/code
;ttyo	equ	0eh	;set to al
;vmod	equ	0fh	;get ax, bh
;kstt	equ	11h	;key status

;rom bios int
;kint	equ	09h	;keybrd int
;vint	equ	10h	;video int
;dint	equ	13h	;disk int

;--------------------------------
;macro
dos	macro	cmd
	mov	ah,cmd
	int	21h
	endm
;
;pusha	macro
;	db	60h
;	endm
;popa	macro
;	db	61h
;	endm
;
;video / rom bios
;romv	macro	cmd
;	mov	ah,cmd
;	int	10h
;	endm
;disk / rom bios
;romd	macro	cmd
;	mov	ah,cmd
;	int	13h
;	endm
;key / rom bios
;romk	macro	cmd
;	mov	ah,cmd
;	int	16h
;	endm
;---------------------------------
start:	cld
	mov	ax,cs
	mov	ds,ax
	mov	es,ax
	mov	ss,ax
	lea	sp,stack
init:	call	crlf
	dos	rdky		;get 1st letter
	cmp	al,esk		;ESC key ?
	je	iniz
	call	cvh2b		;conv hex_ascii to binary
	jc	init
	mov	cl,4
	shl	al,cl		;shift to upper nibble
	mov	bl,al		;temp hold
	dos	rdky		;get 2nd letter
	cmp	al,esk
	je	iniz
	call	cvh2b		;convert
	jc	init
	add	al,bl		;combine 2 digits
	call	dssp		;disp space
ini0:	mov	ah,al		;
	mov	bl,al		;data : ah, bl as temp holder
	shl	ah,1		;left shift for comp '0' following '0'
	xor	al,ah		;get sign of "0 & 0", but "1 & 1" also
	not	al
	mov	bh,al		;temp hold
	mov	al,bl
	and	al,ah		;get sign of "1 & 1"
	not	al
	and	bh,al		;reject sign of "1 & 1"
	shr	bh,1		;right shift for adjust
	mov	al,bh		;make a clock

	xor	dx,dx
	mov	cx,8		;make MFM as "clk_data_clk_data_clk_data ..."
ini1:	rcl	al,1		;output carry from clock
	rcl	dx,1		;include carry bit
	rcl	bl,1		;output carry from data
	rcl	dx,1		;include carry bit
	loop	ini1
	mov	ax,dx
ini2:	cmp	byte ptr cs:[82h],'1'	;request 1st clock ?
	jne	ini3
	or	ax,8000h	;set 1st clock
ini3:	call	ds2bx
	jmp	init

iniz:	xor	al,al
	dos	nrtn		;normal return

mbit	db	00h
;--------------------------------------
;start of sub
;display space
dssp:	push	ax
	push	dx
	mov	dl,' '
	dos	wrch
	pop	dx
	pop	ax
	ret
;--------------------------------------
;put cr & lf
crlf:	push	dx
	mov	dl,cr
	dos	wrch
	mov	dl,lf
	dos	wrch
	pop	dx
	ret
;--------------------------------------
;display 2 hex_ascii on ax(ah+al)
ds2hx:	push	dx
	push	ax
	mov	dl,ah
	dos	wrch
;	romv	ttyo
	pop	ax		;use al
	pop	dx
;display 1 hex_ascii on al
ds1hx:	push	dx
	mov	dl,al
	dos	wrch
;	romv	ttyo
	pop	dx
	ret
;--------------------------------------
;display 2 binary on ax with 4 hex_ascii
ds2bx:	push	ax
	mov	al,ah
	call	cvb2h
	call	ds2hx
	pop	ax
;display 1 binary on al with 2 hex_ascii
ds1bx:	call	cvb2h
	call	ds2hx
	ret
;--------------------------------------
;cnv double/2 hex to bin(1 byte)
cvd2b:	call	cvh2b
	xchg	ah,al
	call	cvh2b
	and	al,0fh
	shl	ah,1
	shl	ah,1
	shl	ah,1
	shl	ah,1
	and	ah,0f0h
	add	al,ah
	ret
;--------------------------------------
;convert hex_ascii to bin (half byte)
cvh2b:	sub	al,'0'
	jc	cvh2e
	cmp	al,10
	jc	cvh21
	sub	al,7
	cmp	al,16
	jc	cvh21
	sub	al,32
	cmc
cvh21:	cmc
cvh2e:	ret
;--------------------------------------
;convert a binary(al) to 2 hex_ascii(ax)
cvb2h:	mov	ah,al
	and	al,0fh
	add	al,'0'
	cmp	al,'9'+1
	jb	cvb21
	add	al,7
cvb21:	shr	ah,1
	shr	ah,1
	shr	ah,1
	shr	ah,1
	add	ah,'0'
	cmp	ah,'9'+1
	jb	cvb2e
	add	ah,7
cvb2e:	clc
	ret
;end of sub
;--------------------------------------
hmsg	db	'make MFM codes Ver 1.0',cr,lf,'$'
stk	dw	128 dup (?)
stack	equ	$

code	ends
	end	start
;--------------------------------------